/*
 * Copyright 2013 Red Hat Inc., Durham, North Carolina.
 * All Rights Reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Authors:
 *      Daniel Kopecek <dkopecek@redhat.com>
 */
#define _GNU_SOURCE
#include <stdio.h>
#include <stdlib.h>
#include <resolv.h>
#include <string.h>
#include <ctype.h>

static const char *sssd_conf_path = "/etc/sssd/sssd.conf";

char *ipa_hostname(void)
{
	static char hname[MAXHOSTNAMELEN+1];
	size_t hname_len = 0;
	char *line = NULL;
	ssize_t line_len = 0;
	size_t line_buflen = 0;
	FILE *fp;

	if ((fp = fopen(sssd_conf_path, "r")) == NULL)
		return NULL;
	while ((line_len = getline(&line, &line_buflen, fp)) > 0) {
		char *keyword_loc;
		if ((keyword_loc = strstr(line, "ipa_hostname")) != NULL) {
			size_t i;
			char *value_loc;
			size_t value_len;

			value_loc = keyword_loc + strlen("ipa_hostname") + 1;
			value_len = line_len - (size_t)(value_loc - line);

			/* Skip spaces and the assignment operator */
			for (i = 0; i < value_len; ++i) {
				if (isspace(value_loc[i]) || value_loc[i] == '=') {
					continue;
				} else {
					break;
				}
			}

			value_loc += i;
			value_len -= i;

			if (value_len <= MAXHOSTNAMELEN) {
				memcpy(hname, value_loc, value_len * sizeof(char));
				free(line);
				fclose(fp);
				hname_len = value_len;
				hname[hname_len] = '\0';
				/* Remove spaces from the end of the string */
				for (i = hname_len - 1; i > 0; --i) {
					if (isspace(hname[i])) {
						hname[i] = '\0';
						--hname_len;
					} else {
						break;
					}
				}
				return hname;
			}
		}
		free(line);
		line = NULL;
	}

	fclose(fp);
	return NULL;
}
